@@ -1,31 +1,8 @@
 # -*- coding: UTF-8 -*-
 #
-# The MIT License
-#
-# Copyright (c) 2011-2016 Felix Schwarz <felix.schwarz@oss.schwarz.eu>
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# I believe the license above is permissible enough so you can actually
-# use/relicense the code in any other project without license proliferation.
-# I'm happy to relicense this code if necessary for inclusion in other free
-# software projects.
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2011-2017, 2019 Felix Schwarz <felix.schwarz@oss.schwarz.eu>
+# Copyright (c)      2012 Robert Buchholz <rbu@goodpoint.de>
 
 # TODO / nice to have
 #  - raising assertions (with message building) should be unified
@@ -34,23 +11,40 @@
 #      see jinja2/debug.py for some code that does such hacks:
 #          https://github.com/mitsuhiko/jinja2/blob/master/jinja2/debug.py
 
-from __future__ import unicode_literals
+from __future__ import absolute_import, unicode_literals, print_function
 
 from contextlib import contextmanager
+from datetime import datetime as DateTime, timedelta as TimeDelta, tzinfo as TZInfo
 import functools
+import os
 from unittest import TestCase
 import sys
 
+try:
+    # Python 3.2+
+    from datetime import timezone as TimeZone
+    UTC = TimeZone.utc
+except ImportError:
+    UTC = None
+
 
 __all__ = ['assert_almost_equals', 'assert_callable', 'assert_contains',
            'assert_dict_contains', 'assert_equals', 'assert_false', 'assert_falseish',
+           'assert_file_exists',
+           'assert_file_not_exists',
            'assert_greater',
+           'assert_is',
+           'assert_is_not',
            'assert_isinstance', 'assert_is_empty', 'assert_is_not_empty',
            'assert_length', 'assert_none',
            'assert_not_raises',
            'assert_not_contains', 'assert_not_none', 'assert_not_equals',
+           'assert_almost_now',
+           'assert_path_exists',
+           'assert_path_not_exists',
            'assert_raises', 'assert_smaller', 'assert_true', 'assert_trueish',
-           'create_spy', 'expect_failure', 'PythonicTestCase', 'SkipTest',
+           'create_spy', 'expect_failure', 'PythonicTestCase',
+           'skip_test', 'skipTest', 'SkipTest',
 ]
 
 # This simple line instructs some test runners (e.g. stock unittest, nosetests,
@@ -117,6 +111,22 @@ def assert_equals(expected, actual, message=None):
         raise AssertionError(default_message)
     raise AssertionError(default_message + ': ' + message)
 
+def assert_is(expr1, expr2, message=None):
+    if expr1 is expr2:
+        return
+    default_message = '%s is not %s' % (repr(expr1), repr(expr2))
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
+def assert_is_not(expr1, expr2, message=None):
+    if expr1 is not expr2:
+        return
+    default_message = '%s is identical to %s' % (repr(expr1), repr(expr2))
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
 def assert_none(actual, message=None):
     assert_equals(None, actual, message=message)
 
@@ -143,7 +153,15 @@ def assert_trueish(actual, message=None):
     raise AssertionError(default_message + ': ' + message)
 
 def assert_length(expected_length, actual_iterable, message=None):
-    assert_equals(expected_length, len(actual_iterable), message=message)
+    length_iterable = _get_length(actual_iterable)
+    assert_equals(expected_length, length_iterable, message=message)
+
+def _get_length(actual_iterable):
+    try:
+        nr_items = len(actual_iterable)
+    except TypeError:
+        nr_items = len(tuple(actual_iterable))
+    return nr_items
 
 def assert_not_equals(expected, actual, message=None):
     if expected != actual:
@@ -197,15 +215,17 @@ def assert_dict_contains(expected_sub_dict, actual_super_dict, message=None):
             raise AssertionError(failure_message)
 
 def assert_is_empty(actual, message=None):
-    if len(actual) == 0:
+    len_actual = _get_length(actual)
+    if len_actual == 0:
         return
-    default_message = '%s is not empty' % (repr(actual))
+    default_message = '%r is not empty (%d items)' % (actual, len_actual)
+    default_message = '%r is not empty' % (actual)
     if message is None:
         raise AssertionError(default_message)
     raise AssertionError(default_message + ': ' + message)
 
 def assert_is_not_empty(actual, message=None):
-    if len(actual) > 0:
+    if _get_length(actual) > 0:
         return
     default_message = '%s is empty' % (repr(actual))
     if message is None:
@@ -249,6 +269,67 @@ def assert_greater(greater, smaller, message=None):
         raise AssertionError(default_message)
     raise AssertionError(default_message + ': ' + message)
 
+def assert_path_exists(path, message=None):
+    if os.path.exists(path):
+        return
+    default_message = "path '%s' does not exist" % (path,)
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
+def assert_path_not_exists(path, message=None):
+    if not os.path.exists(path):
+        return
+    default_message = "path '%s' exists" % (path,)
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
+def assert_file_exists(path, message=None):
+    if os.path.isfile(path):
+        return
+    if os.path.exists(path) and os.path.isdir(path):
+        default_message = "'%s' is a directory" % (path,)
+    else:
+        default_message = "file '%s' does not exist" % (path,)
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
+def assert_file_not_exists(path, message=None):
+    if not os.path.isfile(path):
+        return
+    default_message = "file '%s' exists" % (path,)
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
+def assert_almost_now(datetime, max_delta=None, tz=None, message=None):
+    if max_delta is None:
+        max_delta = TimeDelta(seconds=1)
+    elif not isinstance(max_delta, TimeDelta):
+        max_delta = TimeDelta(seconds=max_delta)
+    has_tz = datetime.tzinfo
+    if tz is None:
+        tz = UTC if has_tz else None
+    elif not has_tz:
+        raise AssertionError('%r is not timezone-aware but tz=%r was passed to assert_almost_equals()' % (datetime, tz))
+
+    now = DateTime.now(tz=tz)
+    if datetime >= now - max_delta:
+        return
+    default_message = '%r is older than %s seconds' % (datetime, _total_seconds(max_delta))
+    if message is None:
+        raise AssertionError(default_message)
+    raise AssertionError(default_message + ': ' + message)
+
+def _total_seconds(timedelta):
+    if hasattr(timedelta, 'total_seconds'):
+        # Python 3.2+
+        return timedelta.total_seconds()
+    return (timedelta.days * 24 * 60 * 60) + timedelta.seconds + (timedelta.microseconds / 1000000)
+
+
 def create_spy(name=None):
     class Spy(object):
         def __init__(self, name=None):
@@ -313,9 +394,11 @@ except ImportError:
             # but at least let's not break as long as the user won't try to
             # use the SkipTest functionality
             SkipTest = AssertionError
-def skipTest(*args, **kwargs):
+def skip_test(*args, **kwargs):
     raise SkipTest(*args, **kwargs)
-
+# ensure that nosetests does not try to run this function as "test"
+skip_test.__test__ = False
+skipTest = skip_test
 
 # --- expect_failure support --------------------------------------------------
 # unittest in Python 2.7 introduced the "expectedFailure" decorator. We should
@@ -346,6 +429,21 @@ def expect_failure(f):
 def add_expected_failure_py(result, test, err):
     result.addSkip(test, str(err))
 
+# --- other helpers -----------------------------------------------------------
+class _UTC(TZInfo):
+    def utcoffset(self, dt):
+        return TimeDelta(seconds=0)
+
+    def dst(self, dt):
+        return TimeDelta(seconds=0)
+
+    def __repr__(self):
+        return '<UTC>'
+
+
+if UTC is None:
+    UTC = _UTC()
+
 # --- unittest.TestCase alternative with pythonic names -----------------------
 from types import MethodType
 
